#include "polygon.hpp"

#include <iostream>
#include <string>
#include <cassert>
#include "Polygon.hpp"

namespace {
void print_info(const std::string& info)
{
    std::cout << info << "\n";
}
}

Polygon::Polygon(size_t size)
    : Polygon(size, 0)
{
    //print_info("C-tor");
}

Polygon::Polygon(const Polygon & rhs)
    : Polygon(rhs.size_, rhs.number_of_points_)
{
    //print_info("copy C-tor");

    for (int i = 0; i < size_; ++i)
    {
        points_[i] = rhs.points_[i];
    }
}

Polygon::Polygon(Polygon && rhs)
    : Polygon()
{
    //print_info("move C-tor");
    swap(*this, rhs);
}

Polygon & Polygon::operator=(const Polygon& rhs)
{
    //print_info("op = ");

    swap(*this, Polygon(rhs));
    return *this;
}

Polygon & Polygon::operator=(Polygon&& rhs)
{
    //print_info("move op = ");

    swap(*this, rhs);
    return *this;
}

void Polygon::push(Point point)
{
    assert(number_of_points_ < size_);
    points_[number_of_points_++] = point;
}

size_t Polygon::numberOfPoints() const
{
    return number_of_points_;
}

Point & Polygon::operator[](size_t index)
{
    assert(index < size_);
    return points_[index];
}

Point Polygon::operator[](size_t index) const
{
    assert(index < size_);
    return points_[index];

}

Polygon::~Polygon()
{
    //print_info("D-tor");

    if (nullptr != points_)
    {
        print_info("zwalnianie pamieci");
        delete[] points_;
    }
}

Polygon::Polygon(size_t size, size_t number_of_elements)
    : size_(size), number_of_points_(number_of_elements), points_(new Point[size_])
{
    // UWAGA :
    // tylko ten konstruktor alokuje pamiec, 
    // wszystkie inne deleguja do niego zadanie przydzielenia pamieci
    print_info("alokacja pamieci");
}

void Polygon::swap(Polygon& lhs, Polygon& rhs)
{
    std::swap(lhs.size_, rhs.size_);
    std::swap(lhs.number_of_points_, rhs.number_of_points_);
    std::swap(lhs.points_, rhs.points_);
}
